#ifndef ENGINE_AMA
#define ENGINE_AMA

#include <algorithm>
#include <cmath>
#include <iostream> 
#include <vector>

// COMPUTE AMA POSTERIOR PROBABILITIES OF CORRECT AND ALL CATEGORIES
//
// m rows
// n columns
// r mean filter response to stimuli    [m x n]
// s filter response stddev to stimuli  [m x n]
// c category indices                   [m x 1]
//////////////////////////////////////
// p    destination of posteriors       [m x 1]
// pAll destination of posteriors       [m x nCtg]

void AMAposteriors (int m, int n, double *r, double *s, double *c, double *p, double *pAll){
    
    // GET TOTAL NUMBER OF CATEGORIES
    const int ctotal = *std::max_element (&c[0], &c[0] + m); // call std library to get pointer to max element in a range
    
    // FOR EACH CANDIDATE STIMULUS
    for (int l = 0; l < m; ++l){
        std::vector<double> num(ctotal); // same as 'const double num[ctotal];' but with better compatibility with old C compilers
        double den = 0.0;
        // FOR EACH CATEGORY
        for (int i = 0; i < ctotal; ++i){
            num[i] = 0.0;
            // FOR EACH STIMULUS
            for (int j = 0; j < m; ++j){
                int cat = c[j]-1;     // c stores ctg from 1 to n. cat must be indexed from 0 to n-1  
                if (cat == i){
                    double sprod  = 1.0f;
                    double expsum = 0.0f;
                    // FOR EACH FILTER
                    for (int t = 0; t < n; ++t){
                        sprod  *= (1/s[j+t*m]);
                        expsum += pow(( (r[l+t*m]-r[j+t*m]) / (s[j+t*m]) ),2);
                    }
                    double prod = sprod*exp(-0.5*expsum);
                    // NUMERATOR AND DENOMINATOR
                    num[cat]   += prod;
                    den        += prod;
                }
            }
        }
        // POSTERIOR PROBABILITY OF CORRECT CATEGORY
        int cat     = c[l]-1;         // c stores ctg from 1 to n. cat must be indexed from 0 to n-1  
        p[l]        = num[cat] / den;     
        
        // POSTERIOR PROBABILITY OF ALL CATEGORIES
        for (int i = 0; i < ctotal; ++i){ 
            pAll[l+i*m] = num[i] / den;   
        }
        
        // DEBUGGING
        // if ((l%400 == 0) && (l < 3200) ){
        //    cout << "l = " << l << ", cat = " << cat << ", r = " << r[l] << ", sigma = " << s[l] << ", num = " << num[cat] << ", den = " << den << ", p = " << p[l] << endl;   
        // }
    }
}

#endif
